@@ -7,12 +7,16 @@
 #[cfg(not(target_os = "openbsd"))]
 use filetime::FileTime;
 use std::fs;
+#[cfg(target_os = "linux")]
+use std::os::unix::ffi::OsStringExt;
 use std::os::unix::fs::{MetadataExt, PermissionsExt};
 #[cfg(not(windows))]
 use std::process::Command;
 #[cfg(any(target_os = "linux", target_os = "android"))]
 use std::thread::sleep;
 use uucore::process::{getegid, geteuid};
+#[cfg(feature = "feat_selinux")]
+use uucore::selinux::get_getfattr_output;
 use uutests::at_and_ucmd;
 use uutests::new_ucmd;
 use uutests::util::{TestScenario, is_ci, run_ucmd_as_root};
@@ -239,6 +243,75 @@ fn test_install_mode_symbolic() {
     assert_eq!(0o100_003_u32, PermissionsExt::mode(&permissions));
 }
 
+#[test]
+fn test_install_mode_comma_separated() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir = "target_dir";
+    let file = "source_file";
+    // Test comma-separated mode like chmod: ug+rwX,o+rX
+    let mode_arg = "--mode=ug+rwX,o+rX";
+
+    at.touch(file);
+    at.mkdir(dir);
+    ucmd.arg(file).arg(dir).arg(mode_arg).succeeds().no_stderr();
+
+    let dest_file = &format!("{dir}/{file}");
+    assert!(at.file_exists(file));
+    assert!(at.file_exists(dest_file));
+    let permissions = at.metadata(dest_file).permissions();
+    // ug+rwX: For files, X only adds execute if file already has execute (it doesn't here, starting at 0)
+    //         So this adds rw to user and group = 0o660
+    // o+rX: For files, X doesn't add execute, so this adds r to others = 0o004
+    // Total: 0o664 for file (0o100_664)
+    assert_eq!(0o100_664_u32, PermissionsExt::mode(&permissions));
+}
+
+#[test]
+fn test_install_mode_comma_separated_directory() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let dir = "test_dir";
+    // Test comma-separated mode for directory creation: ug+rwX,o+rX
+    let mode_arg = "--mode=ug+rwX,o+rX";
+
+    scene
+        .ucmd()
+        .arg("-d")
+        .arg(dir)
+        .arg(mode_arg)
+        .succeeds()
+        .no_stderr();
+
+    assert!(at.dir_exists(dir));
+    let permissions = at.metadata(dir).permissions();
+    // ug+rwX sets user and group to rwx (0o770), o+rX sets others to r-x (0o005)
+    // Total: 0o775 for directory (0o040_775)
+    assert_eq!(0o040_775_u32, PermissionsExt::mode(&permissions));
+}
+
+#[test]
+fn test_install_mode_symbolic_ignore_umask() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let dir = "target_dir";
+    let file = "source_file";
+    let mode_arg = "--mode=+w";
+
+    at.touch(file);
+    at.mkdir(dir);
+    ucmd.arg(file)
+        .arg(dir)
+        .arg(mode_arg)
+        .umask(0o022)
+        .succeeds()
+        .no_stderr();
+
+    let dest_file = &format!("{dir}/{file}");
+    assert!(at.file_exists(file));
+    assert!(at.file_exists(dest_file));
+    let permissions = at.metadata(dest_file).permissions();
+    assert_eq!(0o100_222_u32, PermissionsExt::mode(&permissions));
+}
+
 #[test]
 fn test_install_mode_failing() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -447,6 +520,80 @@ fn test_install_nested_paths_copy_file() {
     assert!(at.file_exists(format!("{dir2}/{file1}")));
 }
 
+#[test]
+fn test_multiple_mode_arguments_override_not_error() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let dir = "source_dir";
+
+    let file = "source_file";
+    let gid = getegid();
+    let uid = geteuid();
+
+    at.touch(file);
+    at.mkdir(dir);
+
+    scene
+        .ucmd()
+        .args(&[
+            file,
+            &format!("{dir}/{file}"),
+            "--owner=invalid_owner",
+            "--owner",
+            &uid.to_string(),
+        ])
+        .succeeds()
+        .no_stderr();
+
+    scene
+        .ucmd()
+        .args(&[
+            file,
+            &format!("{dir}/{file}"),
+            "-o invalid_owner",
+            "-o",
+            &uid.to_string(),
+        ])
+        .succeeds()
+        .no_stderr();
+
+    scene
+        .ucmd()
+        .args(&[file, &format!("{dir}/{file}"), "--mode=999", "--mode=200"])
+        .succeeds()
+        .no_stderr();
+
+    scene
+        .ucmd()
+        .args(&[file, &format!("{dir}/{file}"), "-m 999", "-m 200"])
+        .succeeds()
+        .no_stderr();
+
+    scene
+        .ucmd()
+        .args(&[
+            file,
+            &format!("{dir}/{file}"),
+            "--group=invalid_group",
+            "--group",
+            &gid.to_string(),
+        ])
+        .succeeds()
+        .no_stderr();
+
+    scene
+        .ucmd()
+        .args(&[
+            file,
+            &format!("{dir}/{file}"),
+            "-g invalid_group",
+            "-g",
+            &gid.to_string(),
+        ])
+        .succeeds()
+        .no_stderr();
+}
+
 #[test]
 fn test_install_failing_omitting_directory() {
     let scene = TestScenario::new(util_name!());
@@ -569,7 +716,9 @@ fn test_install_copy_then_compare_file_with_extra_mode() {
         .arg("-m")
         .arg("1644")
         .succeeds()
-        .no_stderr();
+        .stderr_contains(
+            "the --compare (-C) option is ignored when you specify a mode with non-permission bits",
+        );
 
     file2_meta = at.metadata(file2);
     let after_install_sticky = FileTime::from_last_modification_time(&file2_meta);
@@ -615,6 +764,8 @@ fn strip_source_file() -> &'static str {
 
 #[test]
 #[cfg(not(windows))]
+// FIXME test runs in a timeout with macos-latest on x86_64 in the CI
+#[cfg(not(all(target_os = "macos", target_arch = "x86_64")))]
 fn test_install_and_strip() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -638,6 +789,8 @@ fn test_install_and_strip() {
 
 #[test]
 #[cfg(not(windows))]
+// FIXME test runs in a timeout with macos-latest on x86_64 in the CI
+#[cfg(not(all(target_os = "macos", target_arch = "x86_64")))]
 fn test_install_and_strip_with_program() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -929,7 +1082,7 @@ fn test_install_creating_leading_dir_fails_on_long_name() {
         .arg(source)
         .arg(at.plus(target.as_str()))
         .fails()
-        .stderr_contains("failed to create");
+        .stderr_contains("cannot create directory");
 }
 
 #[test]
@@ -1442,6 +1595,13 @@ fn test_install_dir_dot() {
         .arg("-v")
         .succeeds()
         .stdout_contains("creating directory 'dir5/cali'");
+    scene
+        .ucmd()
+        .arg("-d")
+        .arg("dir6/./")
+        .arg("-v")
+        .succeeds()
+        .stdout_contains("creating directory 'dir6'");
 
     let at = &scene.fixtures;
 
@@ -1450,6 +1610,7 @@ fn test_install_dir_dot() {
     assert!(at.dir_exists("dir3"));
     assert!(at.dir_exists("dir4/cal"));
     assert!(at.dir_exists("dir5/cali"));
+    assert!(at.dir_exists("dir6"));
 }
 
 #[test]
@@ -1620,6 +1781,193 @@ fn test_install_compare_option() {
         .stderr_contains("Options --compare and --strip are mutually exclusive");
 }
 
+#[test]
+#[cfg(not(target_os = "openbsd"))]
+fn test_install_compare_basic() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source = "source_file";
+    let dest = "dest_file";
+
+    at.write(source, "test content");
+
+    // First install should copy
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, dest])
+        .succeeds()
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Second install with same mode should be no-op (compare works)
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, dest])
+        .succeeds()
+        .no_stdout();
+
+    // Test that compare works correctly when content actually differs
+    let source2 = "source2";
+    at.write(source2, "different content");
+
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source2, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source2}' -> '{dest}'"));
+
+    // Second install should be no-op since content is now identical
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source2, dest])
+        .succeeds()
+        .no_stdout();
+}
+
+#[test]
+#[cfg(not(any(target_os = "openbsd", target_os = "freebsd")))]
+fn test_install_compare_special_mode_bits() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source = "source_file";
+    let dest = "dest_file";
+
+    at.write(source, "test content");
+
+    // Special mode bits - setgid (tests the core bug fix)
+    // When setgid bit is set, -C should be ignored (always copy)
+    // This tests the bug where b.specified_mode.unwrap_or(0) was used instead of b.mode()
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m2755", source, dest])
+        .succeeds()
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Second install with same setgid mode should ALSO copy (not skip)
+    // because -C option should be ignored when special mode bits are present
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m2755", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Special mode bits - setuid
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m4755", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Second install with setuid should also copy
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m4755", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Special mode bits - sticky bit
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m1755", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Second install with sticky bit should also copy
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m1755", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Back to normal mode - compare should work again
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Second install with normal mode should be no-op
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, dest])
+        .succeeds()
+        .no_stdout();
+}
+
+#[test]
+#[cfg(not(target_os = "openbsd"))]
+fn test_install_compare_group_ownership() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source = "source_file";
+    let dest = "dest_file";
+
+    at.write(source, "test content");
+
+    let user_group = std::process::Command::new("id")
+        .arg("-nrg")
+        .output()
+        .map_or_else(
+            |_| "users".to_string(),
+            |output| String::from_utf8_lossy(&output.stdout).trim().to_string(),
+        ); // fallback group name
+
+    // Install with explicit group
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m664", "-g", &user_group, source, dest])
+        .succeeds()
+        .stdout_contains(format!("'{source}' -> '{dest}'"));
+
+    // Install without group - this should detect that no copy is needed
+    // because the file already has the correct group (user's group)
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m664", source, dest])
+        .succeeds()
+        .no_stdout(); // Should be no-op if group ownership logic is correct
+}
+
+#[test]
+#[cfg(not(target_os = "openbsd"))]
+fn test_install_compare_symlink_handling() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+
+    let source = "source_file";
+    let symlink_dest = "symlink_dest";
+    let target_file = "target_file";
+
+    at.write(source, "test content");
+    at.write(target_file, "test content"); // Same content to test that symlinks are always replaced
+    at.symlink_file(target_file, symlink_dest);
+
+    // Create a symlink as destination pointing to a different file - should always be replaced
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, symlink_dest])
+        .succeeds()
+        .stdout_contains("removed")
+        .stdout_contains(format!("'{source}' -> '{symlink_dest}'"));
+
+    // Even if content would be the same, symlink destination should be replaced
+    // Now symlink_dest is a regular file, so compare should work normally
+    scene
+        .ucmd()
+        .args(&["-Cv", "-m644", source, symlink_dest])
+        .succeeds()
+        .no_stdout(); // Now it's a regular file, so compare should work
+}
+
 #[test]
 // Matches part of tests/install/basic-1
 fn test_t_exist_dir() {
@@ -1811,6 +2159,20 @@ fn test_install_no_target_directory_failing_cannot_overwrite() {
     assert!(!at.dir_exists("dir/file"));
 }
 
+#[test]
+fn test_install_no_target_directory_overwrite_file() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let file = "file";
+    let dest = "dest";
+
+    at.touch(file);
+    scene.ucmd().arg("-T").arg(file).arg(dest).succeeds();
+    scene.ucmd().arg("-T").arg(file).arg(dest).succeeds();
+
+    assert!(!at.dir_exists("dir/file"));
+}
+
 #[test]
 fn test_install_no_target_directory_failing_omitting_directory() {
     let scene = TestScenario::new(util_name!());
@@ -1950,8 +2312,6 @@ fn test_install_no_target_basic() {
 #[test]
 #[cfg(feature = "feat_selinux")]
 fn test_selinux() {
-    use std::process::Command;
-
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
     let src = "orig";
@@ -1961,31 +2321,38 @@ fn test_selinux() {
 
     let args = ["-Z", "--context=unconfined_u:object_r:user_tmp_t:s0"];
     for arg in args {
-        new_ucmd!()
+        let result = new_ucmd!()
             .arg(arg)
             .arg("-v")
             .arg(at.plus_as_string(src))
             .arg(at.plus_as_string(dest))
-            .succeeds()
-            .stdout_contains("orig' -> '");
+            .run();
 
-        let getfattr_output = Command::new("getfattr")
-            .arg(at.plus_as_string(dest))
-            .arg("-n")
-            .arg("security.selinux")
-            .output()
-            .expect("Failed to run `getfattr` on the destination file");
-        println!("{:?}", getfattr_output);
-        assert!(
-            getfattr_output.status.success(),
-            "getfattr did not run successfully: {}",
-            String::from_utf8_lossy(&getfattr_output.stderr)
-        );
+        // Skip test if SELinux is not enabled
+        if result
+            .stderr_str()
+            .contains("SELinux is not enabled on this system")
+        {
+            println!("Skipping SELinux test: SELinux is not enabled");
+            at.remove(&at.plus_as_string(dest));
+            continue;
+        }
+
+        result.success().stdout_contains("orig' -> '");
+
+        // Try to get SELinux context, skip test if getfattr is not available
+        let context_value =
+            std::panic::catch_unwind(|| get_getfattr_output(&at.plus_as_string(dest)));
+
+        let Ok(context_value) = context_value else {
+            println!("Skipping SELinux test: getfattr not available or failed");
+            at.remove(&at.plus_as_string(dest));
+            continue;
+        };
 
-        let stdout = String::from_utf8_lossy(&getfattr_output.stdout);
         assert!(
-            stdout.contains("unconfined_u"),
-            "Expected 'foo' not found in getfattr output:\n{stdout}"
+            context_value.contains("unconfined_u"),
+            "Expected 'unconfined_u' not found in getfattr output:\n{context_value}"
         );
         at.remove(&at.plus_as_string(dest));
     }
@@ -2006,14 +2373,119 @@ fn test_selinux_invalid_args() {
         "--context=nconfined_u:object_r:user_tmp_t:s0",
     ];
     for arg in args {
-        new_ucmd!()
+        let result = new_ucmd!()
             .arg(arg)
             .arg("-v")
             .arg(at.plus_as_string(src))
             .arg(at.plus_as_string(dest))
-            .fails()
-            .stderr_contains("failed to set default file creation");
+            .fails();
+
+        let stderr = result.stderr_str();
+        assert!(
+            stderr.contains("failed to set default file creation")
+                || stderr.contains("SELinux is not enabled on this system"),
+            "Expected stderr to contain either 'failed to set default file creation' or 'SELinux is not enabled on this system', but got: '{stderr}'"
+        );
 
         at.remove(&at.plus_as_string(dest));
     }
 }
+
+#[test]
+#[cfg(feature = "feat_selinux")]
+fn test_selinux_default_context() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    let src = "orig";
+    at.touch(src);
+    let dest = "orig.2";
+
+    let result = new_ucmd!()
+        .arg("-Z")
+        .arg("-v")
+        .arg(at.plus_as_string(src))
+        .arg(at.plus_as_string(dest))
+        .run();
+
+    // Skip test if SELinux is not enabled
+    if result
+        .stderr_str()
+        .contains("SELinux is not enabled on this system")
+    {
+        println!("Skipping SELinux default context test: SELinux is not enabled");
+        return;
+    }
+
+    result.success().stdout_contains("orig' -> '");
+    assert!(at.file_exists(dest));
+}
+
+#[test]
+#[cfg(not(any(target_os = "openbsd", target_os = "freebsd")))]
+fn test_install_compare_with_mode_bits() {
+    let test_cases = [
+        ("4755", "setuid bit", true),
+        ("2755", "setgid bit", true),
+        ("1755", "sticky bit", true),
+        ("7755", "setuid + setgid + sticky bits", true),
+        ("755", "permission-only mode", false),
+    ];
+
+    for (mode, description, should_warn) in test_cases {
+        let scene = TestScenario::new(util_name!());
+        let at = &scene.fixtures;
+        let source = format!("source_file_{mode}");
+        let dest = format!("dest_file_{mode}");
+
+        at.write(&source, "test content");
+
+        let mode_arg = format!("--mode={mode}");
+
+        if should_warn {
+            scene.ucmd().args(&["-C", &mode_arg, &source, &dest])
+                .succeeds()
+                .stderr_contains("the --compare (-C) option is ignored when you specify a mode with non-permission bits");
+        } else {
+            scene
+                .ucmd()
+                .args(&["-C", &mode_arg, &source, &dest])
+                .succeeds()
+                .no_stderr();
+
+            // Test second install should be no-op due to -C
+            scene
+                .ucmd()
+                .args(&["-C", &mode_arg, &source, &dest])
+                .succeeds()
+                .no_stderr();
+        }
+
+        assert!(
+            at.file_exists(&dest),
+            "Failed to create dest file for {description}"
+        );
+    }
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_install_non_utf8_paths() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    let source_filename = std::ffi::OsString::from_vec(vec![0xFF, 0xFE]);
+    let dest_dir = "target_dir";
+
+    std::fs::write(at.plus(&source_filename), b"test content").unwrap();
+    at.mkdir(dest_dir);
+
+    ucmd.arg(&source_filename).arg(dest_dir).succeeds();
+
+    // Test with trailing slash and directory creation (-D flag)
+    let (at, mut ucmd) = at_and_ucmd!();
+    let source_file = "source.txt";
+    let mut target_path = std::ffi::OsString::from_vec(vec![0xFF, 0xFE, b'd', b'i', b'r']);
+    target_path.push("/target.txt");
+
+    at.touch(source_file);
+
+    ucmd.arg("-D").arg(source_file).arg(&target_path).succeeds();
+}
