@@ -3,16 +3,16 @@
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
 
-// spell-checker:ignore abcdefghijklmnopqrstuvwxyz Anone
+// spell-checker:ignore abcdefghijklmnopqrstuvwxyz Anone fdbb littl bfloat
 
 #[cfg(unix)]
 use std::io::Read;
+#[cfg(target_os = "linux")]
+use std::path::Path;
 
 use unindent::unindent;
-use uutests::at_and_ucmd;
-use uutests::new_ucmd;
 use uutests::util::TestScenario;
-use uutests::util_name;
+use uutests::{at_and_ucmd, new_ucmd, util_name};
 
 // octal dump of 'abcdefghijklmnopqrstuvwxyz\n'
 static ALPHA_OUT: &str = "
@@ -21,6 +21,27 @@ static ALPHA_OUT: &str = "
         0000033
         ";
 
+fn erange_message() -> String {
+    let err = std::io::Error::from_raw_os_error(libc::ERANGE);
+    let msg = err.to_string();
+    msg.split(" (os error").next().unwrap_or(&msg).to_string()
+}
+
+fn run_skip_across_inputs(files: &[(&str, &str)], skip: u64, expected: &str) {
+    let (at, mut ucmd) = at_and_ucmd!();
+    for (name, contents) in files {
+        at.write(name, contents);
+    }
+
+    ucmd.arg("-c").arg("-j").arg(skip.to_string()).arg("-An");
+
+    for (name, _) in files {
+        ucmd.arg(name);
+    }
+
+    ucmd.succeeds().stdout_only(expected);
+}
+
 #[test]
 fn test_invalid_arg() {
     new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
@@ -32,39 +53,30 @@ fn test_file() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
     at.write("test", "abcdefghijklmnopqrstuvwxyz\n");
-    scene
-        .ucmd()
-        .arg("--endian=little")
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
-    scene
-        .ucmd()
-        .arg("--endian=littl") // spell-checker:disable-line
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
-    scene
-        .ucmd()
-        .arg("--endian=l")
-        .arg("test")
-        .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+
+    for arg in ["--endian=little", "--endian=littl", "--endian=l"] {
+        scene
+            .ucmd()
+            .arg(arg)
+            .arg("test")
+            .succeeds()
+            .stdout_only(unindent(ALPHA_OUT));
+    }
+
     // Ensure that default format matches `-t o2`, and that `-t` does not absorb file argument
     scene
         .ucmd()
         .arg("--endian=little")
         .arg("-t")
         .arg("o2")
-        .arg("test");
+        .arg("test")
+        .succeeds()
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od can read 2 files and concatenate the contents
 #[test]
-fn test_2files() {
+fn test_two_files() {
     let (at, mut ucmd) = at_and_ucmd!();
     at.write("test1", "abcdefghijklmnop");
     at.write("test2", "qrstuvwxyz\n"); // spell-checker:disable-line
@@ -72,15 +84,13 @@ fn test_2files() {
         .arg("test1")
         .arg("test2")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od gives non-0 exit val for filename that doesn't exist.
 #[test]
-fn test_no_file() {
-    let (_at, mut ucmd) = at_and_ucmd!();
-    ucmd.arg("}surely'none'would'thus'a'file'name").fails();
+fn test_non_existing_file() {
+    new_ucmd!().arg("non_existing_file").fails();
 }
 
 // Test that od reads from stdin instead of a file
@@ -91,8 +101,7 @@ fn test_from_stdin() {
         .arg("--endian=little")
         .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 // Test that od reads from stdin and also from files
@@ -110,8 +119,7 @@ fn test_from_mixed() {
         .arg("test-3")
         .run_piped_stdin(data2.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(ALPHA_OUT));
+        .stdout_only(unindent(ALPHA_OUT));
 }
 
 #[test]
@@ -122,8 +130,7 @@ fn test_multiple_formats() {
         .arg("-b")
         .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000000   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p
                     141 142 143 144 145 146 147 150 151 152 153 154 155 156 157 160
@@ -151,8 +158,7 @@ fn test_dec() {
         .arg("-s")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -171,8 +177,7 @@ fn test_hex16() {
         .arg("-x")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -189,8 +194,33 @@ fn test_hex32() {
         .arg("-X")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
+}
+
+// Regression: 16-bit IEEE half should print with canonical precision (no spurious digits)
+#[test]
+fn test_float16_compact() {
+    let input: [u8; 4] = [0x3c, 0x00, 0x3c, 0x00]; // two times 1.0 in big-endian half
+    new_ucmd!()
+        .arg("--endian=big")
+        .arg("-An")
+        .arg("-tfH")
+        .run_piped_stdin(&input[..])
+        .success()
+        .stdout_only("               1               1\n");
+}
+
+// Regression: 16-bit bfloat should print with canonical precision (no spurious digits)
+#[test]
+fn test_bfloat16_compact() {
+    let input: [u8; 4] = [0x3f, 0x80, 0x3f, 0x80]; // two times 1.0 in big-endian bfloat16
+    new_ucmd!()
+        .arg("--endian=big")
+        .arg("-An")
+        .arg("-tfB")
+        .run_piped_stdin(&input[..])
+        .success()
+        .stdout_only("               1               1\n");
 }
 
 #[test]
@@ -206,7 +236,7 @@ fn test_f16() {
     ]; // 0x8400 -6.104e-5
     let expected_output = unindent(
         "
-            0000000       1.0000000               0              -0             inf
+            0000000               1               0              -0             inf
             0000010            -inf             NaN   -6.1035156e-5
             0000016
             ",
@@ -217,8 +247,7 @@ fn test_f16() {
         .arg("-w8")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -234,7 +263,7 @@ fn test_fh() {
     ]; // 0x8400 -6.1035156e-5
     let expected_output = unindent(
         "
-            0000000       1.0000000               0              -0             inf
+            0000000               1               0              -0             inf
             0000010            -inf             NaN   -6.1035156e-5
             0000016
         ",
@@ -245,8 +274,7 @@ fn test_fh() {
         .arg("-w8")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -262,7 +290,7 @@ fn test_fb() {
     ]; // -6.1035156e-5
     let expected_output = unindent(
         "
-            0000000       1.0000000               0              -0             inf
+            0000000               1               0              -0             inf
             0000010            -inf             NaN   -6.1035156e-5
             0000016
         ",
@@ -273,8 +301,7 @@ fn test_fb() {
         .arg("-w8")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -300,8 +327,7 @@ fn test_f32() {
         .arg("-f")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -329,8 +355,7 @@ fn test_f64() {
         .arg("-F")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -340,8 +365,7 @@ fn test_multibyte() {
         .args(&["-t", "c"])
         .run_piped_stdin(input.as_bytes())
         .success()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000000 342 200 231 342 200 220 313 206 342 200 230 313 234 350 252 236
             0000020 360 237 231 202 342 234 205 360 237 220 266 360 235 233 221   U
@@ -368,8 +392,7 @@ fn test_width() {
         .arg("-v")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -394,22 +417,29 @@ fn test_invalid_width() {
 
 #[test]
 fn test_zero_width() {
-    let input: [u8; 4] = [0x00, 0x00, 0x00, 0x00];
-    let expected_output = unindent(
-        "
-            0000000 000000
-            0000002 000000
-            0000004
-            ",
-    );
-
     new_ucmd!()
         .arg("-w0")
-        .arg("-v")
-        .run_piped_stdin(&input[..])
-        .success()
-        .stderr_is_bytes("od: warning: invalid width 0; using 2 instead\n".as_bytes())
-        .stdout_is(expected_output);
+        .arg("-An")
+        .fails_with_code(1)
+        .stderr_only("od: invalid -w argument '0'\n");
+}
+
+#[test]
+fn test_negative_width_argument() {
+    new_ucmd!()
+        .arg("-w-1")
+        .arg("-An")
+        .fails_with_code(1)
+        .stderr_only("od: invalid -w argument '-1'\n");
+}
+
+#[test]
+fn test_non_numeric_width_argument() {
+    new_ucmd!()
+        .arg("-ww")
+        .arg("-An")
+        .fails_with_code(1)
+        .stderr_only("od: invalid -w argument 'w'\n");
 }
 
 #[test]
@@ -425,8 +455,43 @@ fn test_width_without_value() {
         .arg("-w")
         .run_piped_stdin(&input[..])
         .success()
-        .no_stderr()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
+}
+
+#[test]
+fn test_very_wide_ascii_output() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.write("data-a", "x");
+    ucmd.arg("-a")
+        .arg("-w65537")
+        .arg("-An")
+        .arg("data-a")
+        .succeeds()
+        .stdout_only("   x\n");
+}
+
+#[test]
+fn test_very_wide_char_output() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.write("data-c", "x");
+    ucmd.arg("-c")
+        .arg("-w65537")
+        .arg("-An")
+        .arg("data-c")
+        .succeeds()
+        .stdout_only("   x\n");
+}
+
+#[test]
+fn test_very_wide_hex_byte_output() {
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.write_bytes("data-x", &[0x42]);
+    ucmd.arg("-tx1")
+        .arg("-w65537")
+        .arg("-An")
+        .arg("data-x")
+        .succeeds()
+        .stdout_only(" 42\n");
 }
 
 #[test]
@@ -456,9 +521,8 @@ fn test_suppress_duplicates() {
         .arg("-O")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -482,9 +546,8 @@ fn test_big_endian() {
         .arg("-X")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(&expected_output);
+        .stdout_only(&expected_output);
     new_ucmd!()
         .arg("--endian=b")
         .arg("-F")
@@ -492,9 +555,8 @@ fn test_big_endian() {
         .arg("-X")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -518,9 +580,8 @@ fn test_alignment_Xxa() {
         .arg("-x")
         .arg("-a")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -542,9 +603,8 @@ fn test_alignment_Fx() {
         .arg("-F")
         .arg("-x")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -570,9 +630,8 @@ fn test_max_uint() {
         .arg("-Dd")
         .arg("--format=u1")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -593,9 +652,8 @@ fn test_hex_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -616,9 +674,8 @@ fn test_dec_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -632,9 +689,8 @@ fn test_no_offset() {
         .arg("-X")
         .arg("-X")
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(expected_output);
+        .stdout_only(expected_output);
 }
 
 #[test]
@@ -642,6 +698,53 @@ fn test_invalid_offset() {
     new_ucmd!().arg("-Ab").fails();
 }
 
+#[test]
+fn test_invalid_traditional_offsets_are_filenames() {
+    let cases = [("++0", "++0"), ("+-0", "+-0"), ("+ 0", "'+ 0'")];
+
+    for (input, display) in cases {
+        new_ucmd!()
+            .arg(input)
+            .fails_with_code(1)
+            .stderr_only(format!("od: {display}: No such file or directory\n"));
+    }
+
+    new_ucmd!()
+        .arg("--")
+        .arg("-0")
+        .fails_with_code(1)
+        .stderr_only("od: -0: No such file or directory\n");
+}
+
+#[test]
+fn test_traditional_offset_overflow_diagnosed() {
+    let erange = erange_message();
+    let long_octal = "7".repeat(255);
+    let long_decimal = format!("{}.", "9".repeat(254));
+    let long_hex = format!("0x{}", "f".repeat(253));
+
+    new_ucmd!()
+        .arg("-")
+        .arg(&long_octal)
+        .pipe_in(Vec::<u8>::new())
+        .fails_with_code(1)
+        .stderr_only(format!("od: {long_octal}: {erange}\n"));
+
+    new_ucmd!()
+        .arg("-")
+        .arg(&long_decimal)
+        .pipe_in(Vec::<u8>::new())
+        .fails_with_code(1)
+        .stderr_only(format!("od: {long_decimal}: {erange}\n"));
+
+    new_ucmd!()
+        .arg("-")
+        .arg(&long_hex)
+        .pipe_in(Vec::<u8>::new())
+        .fails_with_code(1)
+        .stderr_only(format!("od: {long_hex}: {erange}\n"));
+}
+
 #[test]
 fn test_empty_offset() {
     new_ucmd!()
@@ -670,9 +773,8 @@ fn test_skip_bytes() {
         .arg("-c")
         .arg("--skip-bytes=5")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000005   f   g   h   i   j   k   l   m   n   o   p   q
             0000021
@@ -687,9 +789,8 @@ fn test_skip_bytes_hex() {
         .arg("-c")
         .arg("--skip-bytes=0xB")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000013   l   m   n   o   p   q
             0000021
@@ -699,9 +800,8 @@ fn test_skip_bytes_hex() {
         .arg("-c")
         .arg("--skip-bytes=0xE")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000016   o   p   q
             0000021
@@ -709,6 +809,59 @@ fn test_skip_bytes_hex() {
         ));
 }
 
+#[test]
+fn test_skip_bytes_consumes_single_input() {
+    run_skip_across_inputs(&[("g", "a")], 1, "");
+}
+
+#[test]
+fn test_skip_bytes_consumes_two_inputs() {
+    run_skip_across_inputs(&[("g", "a"), ("h", "b")], 2, "");
+}
+
+#[test]
+fn test_skip_bytes_consumes_three_inputs() {
+    run_skip_across_inputs(&[("g", "a"), ("h", "b"), ("i", "c")], 3, "");
+}
+
+#[test]
+fn test_skip_bytes_prints_after_consuming_multiple_inputs() {
+    run_skip_across_inputs(
+        &[("g", "a"), ("h", "b"), ("i", "c"), ("j", "d")],
+        3,
+        "   d\n",
+    );
+}
+
+#[cfg(target_os = "linux")]
+#[test]
+fn test_skip_bytes_proc_file_without_seeking() {
+    let proc_path = Path::new("/proc/version");
+    if !proc_path.exists() {
+        return;
+    }
+
+    let Ok(contents) = std::fs::read(proc_path) else {
+        return;
+    };
+
+    if contents.is_empty() {
+        return;
+    }
+
+    let (at, mut ucmd) = at_and_ucmd!();
+    at.write("after", "e");
+
+    ucmd.arg("-An")
+        .arg("-c")
+        .arg("-j")
+        .arg(contents.len().to_string())
+        .arg(proc_path)
+        .arg("after")
+        .succeeds()
+        .stdout_only("   e\n");
+}
+
 #[test]
 fn test_skip_bytes_error() {
     let input = "12345";
@@ -765,9 +918,8 @@ fn test_ascii_dump() {
     new_ucmd!()
         .arg("-tx1zacz")    // spell-checker:disable-line
         .run_piped_stdin(&input[..])
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000000  00  01  0a  0d  10  1f  20  61  62  63  7d  7e  7f  80  90  a0  >...... abc}~....<
                     nul soh  nl  cr dle  us  sp   a   b   c   }   ~ del nul dle  sp
@@ -793,8 +945,7 @@ fn test_filename_parsing() {
         .arg("--")
         .arg("-f")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             000000   m   i   n   u   s  sp   l   o   w   e   r   c   a   s   e  sp
             000010   f  nl
@@ -810,9 +961,8 @@ fn test_stdin_offset() {
         .arg("-c")
         .arg("+5")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             "
             0000005   f   g   h   i   j   k   l   m   n   o   p   q
             0000021
@@ -820,6 +970,24 @@ fn test_stdin_offset() {
         ));
 }
 
+#[test]
+fn test_traditional_decimal_dot_offset() {
+    new_ucmd!()
+        .arg("+1.")
+        .pipe_in("a")
+        .succeeds()
+        .stdout_only("0000001\n");
+}
+
+#[test]
+fn test_traditional_dot_block_offset() {
+    new_ucmd!()
+        .arg("+1.b")
+        .pipe_in(vec![b'a'; 512])
+        .succeeds()
+        .stdout_only("0001000\n");
+}
+
 #[test]
 fn test_file_offset() {
     new_ucmd!()
@@ -828,8 +996,7 @@ fn test_file_offset() {
         .arg("-f")
         .arg("10")
         .succeeds()
-        .no_stderr()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000010   w   e   r   c   a   s   e       f  \n
             0000022
@@ -849,9 +1016,8 @@ fn test_traditional() {
         .arg("10")
         .arg("0")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000010 (0000000)   i   j   k   l   m   n   o   p   q
                                 i   j   k   l   m   n   o   p   q
@@ -870,9 +1036,8 @@ fn test_traditional_with_skip_bytes_override() {
         .arg("-c")
         .arg("0")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000000   a   b   c   d   e   f   g   h   i   j   k   l   m   n   o   p
             0000020
@@ -889,9 +1054,8 @@ fn test_traditional_with_skip_bytes_non_override() {
         .arg("--skip-bytes=10")
         .arg("-c")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             0000012   k   l   m   n   o   p
             0000020
@@ -923,9 +1087,8 @@ fn test_traditional_only_label() {
         .arg("10")
         .arg("0x10")
         .run_piped_stdin(input.as_bytes())
-        .no_stderr()
         .success()
-        .stdout_is(unindent(
+        .stdout_only(unindent(
             r"
             (0000020)   i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x
                         i   j   k   l   m   n   o   p   q   r   s   t   u   v   w   x
@@ -975,3 +1138,124 @@ fn test_od_invalid_bytes() {
             .stderr_only(format!("od: {option} argument '{BIG_SIZE}' too large\n"));
     }
 }
+
+#[test]
+fn test_od_options_after_filename() {
+    let file = "test";
+    let (at, mut ucmd) = at_and_ucmd!();
+    let input: [u8; 4] = [0x68, 0x1c, 0xbb, 0xfd];
+    at.write_bytes(file, &input);
+
+    ucmd.arg(file)
+        .arg("-v")
+        .arg("-An")
+        .arg("-t")
+        .arg("x2")
+        .succeeds()
+        .stdout_only(" 1c68 fdbb\n");
+}
+
+#[test]
+fn test_od_strings_option() {
+    // Test -S option: output strings of at least N graphic chars
+
+    // Test -S0: output all null-terminated strings regardless of length
+    new_ucmd!()
+        .arg("-S0")
+        .pipe_in(b"hello\x00world\x00")
+        .succeeds()
+        .stdout_is("0000000 hello\n0000006 world\n");
+
+    // Test -S0 with single character strings
+    new_ucmd!()
+        .arg("-S0")
+        .pipe_in(b"a\x00b\x00cd\x00")
+        .succeeds()
+        .stdout_is("0000000 a\n0000002 b\n0000004 cd\n");
+
+    // Test with null-terminated strings
+    new_ucmd!()
+        .arg("-S3")
+        .pipe_in(b"\x01hello\x00world\x00ab\x00")
+        .succeeds()
+        .stdout_is("0000001 hello\n0000007 world\n");
+
+    // Test with -S10 to show only strings with minimum length
+    new_ucmd!()
+        .arg("-S10")
+        .pipe_in(b"\x01          \x00          \x00")
+        .succeeds()
+        .stdout_is("0000001           \n0000014           \n");
+
+    // Test with unterminated string at EOF (should not output)
+    new_ucmd!()
+        .arg("-S10")
+        .pipe_in(b"          ")
+        .succeeds()
+        .no_output();
+
+    // Test with -N limit and pending string (should output even without null)
+    let expected = "0000000   \n";
+    new_ucmd!()
+        .arg("-N2")
+        .arg("-S1")
+        .pipe_in("  ")
+        .succeeds()
+        .stdout_is(expected);
+
+    // Test with -N limit and string too short
+    new_ucmd!()
+        .arg("-N11")
+        .arg("-S11")
+        .pipe_in(b"          \x00")
+        .succeeds()
+        .no_output();
+
+    // Test with no address prefix (-An)
+    new_ucmd!()
+        .arg("-S3")
+        .arg("-An")
+        .pipe_in(b"hello\x00world\x00")
+        .succeeds()
+        .stdout_is("hello\nworld\n");
+}
+
+#[test]
+fn test_od_eintr_handling() {
+    // Test that od properly handles EINTR (ErrorKind::Interrupted) during read operations
+    // This verifies the signal interruption retry logic in PartialReader implementation
+    // This verifies the signal interruption retry logic in PartialReader implementation
+
+    let file = "test_eintr";
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    // Create test data
+    let test_data = [0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x0a]; // "Hello\n"
+    at.write_bytes(file, &test_data);
+
+    // Test that od can handle interrupted reads during file processing
+    // The EINTR handling in PartialReader should retry and complete successfully
+    ucmd.arg(file)
+        .arg("-t")
+        .arg("c")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("H"); // Should contain 'H' from "Hello"
+
+    // Test with skip and offset options to exercise different code paths
+    // Create a new command instance to avoid "already run this UCommand" error
+    let (at, mut ucmd2) = at_and_ucmd!();
+    at.write_bytes(file, &test_data);
+
+    ucmd2
+        .arg(file)
+        .arg("-j")
+        .arg("1")
+        .arg("-N")
+        .arg("3")
+        .arg("-t")
+        .arg("c")
+        .succeeds()
+        .no_stderr()
+        .stdout_contains("e"); // Should contain 'e' from "ello"
+}
